package ucot.input;
/**
 * UseCase step class. Contains step's text and relation to another usecase (sub-usecase)
 * @author vevijopi
 *
 */
public class UseCaseStep {
	/**
	 * Step description
	 */
	private String step = new String();
	
	/**
	 * Sub usecase for this step
	 */
	private UseCase subUseCase = null;
	
	/**
	 * Used to connect steps to sub-usecases.
	 * id of another usecase (loaded from file). It is used to connect
	 * these two together.
	 */
	private String subUseCaseId = null;
	
	
	public UseCaseStep(String step, UseCase a) {
		this.step = step;
		subUseCase = a;
	}
	
	/**
	 * Set sub usecase's id
	 * @param id id of a subusecase for this step
	 */
	public void setSubUseCaseId(String id) {
		subUseCaseId = id;
	}
	
	/**
	 * Returns sub usecase's id
	 * @return usecase id
	 */
	public String getSubUseCaseId() {
		return subUseCaseId;
	}
	
	/**
	 * Sets step's description
	 * @param step description
	 */
	public void setStep(String step) {
		this.step = step;
	}
	
	/**
	 * Returns step's description
	 * @return description 
	 */
	public String getStep() {
		return getStep(false);
	}
	
	/**
	 * Returns step's description
	 * @param showSubUsecaseName should the subusecase's name be shown aswell
	 * @return description 
	 */
	public String getStep(boolean showSubUsecaseName) {
		if (!showSubUsecaseName || subUseCase == null)
		return step;
		return step + " (" + subUseCase.getName() + ")";
	}
	
	
	/**
	 * returns sub usecase
	 * @return sub usecase
	 */
	public UseCase getSubUseCase() {
		return subUseCase;
	}
	
	/**
	 * Sets sub usecase
	 * @param usecase
	 */
	public void setSubUseCase(UseCase usecase) {
		subUseCase = usecase;
	}
	
	public boolean equals(Object o){
		if (o instanceof UseCaseStep) {
			UseCaseStep other = (UseCaseStep) o;
			return this.getStep().equals(other.getStep());
		}
		return false;
	}
	
	/**
	 * returns this step as a string aswell as subusecase if one exists
	 */
	public String toString() {
		if (subUseCase == null)
			return step;
		return step + " -> " + subUseCase.toString();
	}
}
