package ucot.model;

public class NoSuchAttributeException extends AnalyzeModelException {

	private static final long serialVersionUID = 8915129262644339201L;

	/** The name of the entity.*/
	String entityName;
	
	/** The name of the method */
	String attributeName;
	
	/**
	 * <p>Builds message for the error.
	 * @param entityName The entity name used in the error.
	 * @return The error message string.
	 */
	private static String buildMessage(String entityName, String attributeName){
		return "No such attribute '"+attributeName+"' in entity '"+entityName+"'.";
	}
	
	private void setInternals(String entityName, String attributeName){
		this.entityName = entityName;
		this.attributeName = attributeName;
	}
	
	public NoSuchAttributeException(String entityName, String attributeName) {
		super(buildMessage(entityName,attributeName));
		setInternals(entityName,attributeName);
	}
	
	public NoSuchAttributeException(String entityName, String attributeName, String message) {
		super(buildMessage(entityName,attributeName)+" " + message);
		setInternals(entityName,attributeName);
	}

	public NoSuchAttributeException(String entityName, String attributeName, Throwable cause) {
		super(buildMessage(entityName, attributeName), cause);
		setInternals(entityName,attributeName);
	}
	
	public NoSuchAttributeException(String entityName, String attributeName, String message, Throwable cause) {
		super(buildMessage(entityName, attributeName)+" "+message, cause);
		setInternals(entityName,attributeName);
	}

}
