package ucot.model;
/**
 * <pre>public class <b>NoSuchEntityException</b></pre>
 * <p>Error which is thrown if no entity of given name
 * is found.</p>
 * @author pajumasu
 *
 */
public class NoSuchEntityException extends AnalyzeModelException {

	
	private static final long serialVersionUID = 3810508935149009434L;

	/** The name of the entity.*/
	String entityName;
	
	/**
	 * <p>Builds message for the error.
	 * @param entityName The entity name used in the error.
	 * @return The error message string.
	 */
	protected static String buildMessage(String entityName){
		return "No such entity called '"+entityName+"'.";
	}
	
	public NoSuchEntityException(String entityName) {
		super(buildMessage(entityName));
		this.entityName = entityName;
	}
	
	public NoSuchEntityException(String entityName, String message) {
		super(buildMessage(entityName)+" " + message);
		this.entityName = entityName;
	}

	public NoSuchEntityException(String entityName, Throwable cause) {
		super(buildMessage(entityName), cause);
		this.entityName = entityName;
	}
	
	public NoSuchEntityException(String entityName, String message, Throwable cause) {
		super(buildMessage(entityName)+" "+message, cause);
		this.entityName = entityName;
	}

}
