package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.filechooser.FileFilter;

import ucot.model.AnalyzeModel;
import ucot.model.Updation;
import ucot.model.Updation.UpdationType;
import ucot.utils.CustomFileFilter;
/**
 * This is simple window to show modification log for
 * an analyzemodel.
 * @author pajumasu
 *
 */
public class ModificationLogWindow extends JFrame {

	private static final long serialVersionUID = -711720267848958080L;

	/**
	 * The model that we are interested in.
	 */
	AnalyzeModel model = null;
	
	/**
	 * Text area for textual information about the modifications.
	 */
	JTextArea text;
	
	/**
	 * The last index of the updation element added to the
	 * text field.
	 * Using this we dont rewrite the whole text again
	 * when asked to update.
	 */
	int last = 0;
	
	
	final static String CLOSE_CAPTION= "Close";
	final static String SAVE_AS_CAPTION= "Save as..";
	final static String UPDATE_CAPTION= "Update";
	
	final static String CLOSE_ACTION= "CLOSE";
	final static String SAVE_AS_ACTION= "SAVE_AS";
	final static String UPDATE_ACTION= "UPDATE";
	
	final static String TEXT_DESCRIPTION = "Text file";
	final static String[] TEXT_EXTENSIONS = {"txt"};
	
	JFileChooser fileChooser;
	
	ActionListener actionListener = new ActionListener(){
		
		
		private void closeAction(){
			close();
		}
		private void saveAsAction(){
			if (saveAs()) close();
		}
		private void updateAction(){
			update();
		}
		public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand();
			if (cmd == null){return;}
			else if (cmd.equals(CLOSE_ACTION)){closeAction();}
			else if (cmd.equals(SAVE_AS_ACTION)){saveAsAction();}
			else if (cmd.equals(UPDATE_ACTION)){updateAction();}
			
		}
		
	};
	
	public ModificationLogWindow(AnalyzeModel model){
		this.model = model;
		
		setTitle("Modification log");		
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		text = new JTextArea();
		
		JScrollPane textScrollPane = new JScrollPane(text);
		textScrollPane.setAutoscrolls(true);
		
		JPanel buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new FlowLayout());
		
		
		JButton closeButton = new JButton("Close");
		closeButton.setActionCommand(CLOSE_ACTION);
		closeButton.addActionListener(actionListener);
		
		JButton saveButton = new JButton("Save as..");
		saveButton.setActionCommand(SAVE_AS_ACTION);
		saveButton.addActionListener(actionListener);
		
		JButton updateButton = new JButton("Update");
		updateButton.setActionCommand(UPDATE_ACTION);
		updateButton.addActionListener(actionListener);
		
		JFileChooser fileChooser = new JFileChooser();
		
		buttonsPanel.add(updateButton);
		buttonsPanel.add(saveButton);
		buttonsPanel.add(closeButton);
		
		
		
		
		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(textScrollPane,BorderLayout.CENTER);
		this.getContentPane().add(buttonsPanel,BorderLayout.SOUTH);
		
		
		
		
		this.setPreferredSize(new Dimension(400,400));
		
		update();
		
		this.pack();
		setVisible(true);
	}
	
	/**
	 * Updates the view using the current model.
	 */
	public void update(){
		update(model);
	}
	/**
	 * Updates the view.
	 * @param model The model whitch modifications we like to see.
	 */
	public void update(AnalyzeModel model){
		if (model != this.model){
			last = 0;
			text.setText("");
		}
		this.model = model;
		
		List<Updation> updations = model.getEditor().getUpdations();
		for(int i = last; i<updations.size(); i++){
			Updation updation = updations.get(i);
			if (updation.getType() == UpdationType.READY) {
				text.append("\n");
			} else if (updation.getType() == UpdationType.MODIFICATION_STARTED) {
				// do nothing..
				// this is just to filter MODIFICATION_STARTED type from log..
			} else {
				String[] ref = updation.getReferences();
				StringBuffer sb = new StringBuffer();
				sb.append("[");
				for(String str : ref){
					sb.append(str);
					sb.append(",");
				}
				if (sb.length()>1) sb.setLength(sb.length()-1);
				sb.append("]");
				text.append(updation.getType().toString());
				text.append(" ");
				text.append(updation.getDataType().toString());
				text.append(" ");
				text.append(sb.toString());
				text.append("\n");
			}
			
		}
		last = updations.size();
	}
	
	
	public void close(){
		this.setVisible(false);
		this.dispose();
	}
	public boolean saveAs(){
		if (fileChooser == null) fileChooser =  new JFileChooser(".");
		
		Vector<CustomFileFilter> fileFilters = new Vector<CustomFileFilter>();
		fileFilters.add(new
				CustomFileFilter(TEXT_DESCRIPTION,
						TEXT_EXTENSIONS));
		fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
		fileChooser.setAcceptAllFileFilterUsed(true);
		for ( FileFilter filter : fileFilters ) {
			fileChooser.setFileFilter(filter);
		}
		
		int returnValue = fileChooser.showSaveDialog(this);
		if (returnValue == JFileChooser.APPROVE_OPTION){
			File file = fileChooser.getSelectedFile();
			
			if ( file.exists() ) {
				int overwrite = Integer.parseInt(GUIUtils.questionDialog(this,
						"File " + file + " already exists.\n" +
				"Do you want to replace it?").toString());
				if ( overwrite == JOptionPane.NO_OPTION ) return false;
			}
			
			// Do the save.
			try{
				OutputStream out = new FileOutputStream(file);
				String textStr = text.getText();
				out.write(textStr.getBytes());
				out.close();
				return true;
			}catch(Exception e){
				GUIUtils.printError(this, e.getMessage());
			}
						

		}
		return false;
	}
}
