package ucot.ui.gui.dialog.entitytab;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Logger;

import ucot.Messages;
import ucot.model.AnalyzeModel;
import ucot.model.AnalyzeModelException;
import ucot.model.ModelEditor;
import ucot.model.NoSuchEntityException;

//TODO comment me
public class MethodsPanel extends JTableEntityPropertiesTab {
	
	
	private static final long serialVersionUID = -5071995337843998163L;	
	
	/** Log message for informing creation of the new methods. */
	protected final static String NEW_METHODS_LOG_MESSAGE = Messages.getString("MethodsPanel.NEW_METHODS_LOG_MESSAGE");	
	/** Log message for informing deletion of removed methods. */
	protected final static String DELETED_METHODS_LOG_MESSAGE = Messages.getString("MethodsPanel.DELETED_METHODS_LOG_MESSAGE");	
	/** Log message for informing change of methods. */
	protected final static String CHANGED_METHODS_LOG_MESSAGE = Messages.getString("MethodsPanel.CHANGED_METHODS_LOG_MESSAGE");
	
	/** Text for new method button. */
	protected static String NEW_METHOD_BUTTON = Messages.getString("EntityPropertiesDialog.NEW_METHOD_BUTTON");
	/** Text for delete method button. */
	protected static String DELETE_METHOD_BUTTON = Messages.getString("EntityPropertiesDialog.DELETE_METHOD_BUTTON");
	
	
	protected final static String NEW_METHOD_BUTTON_ACTION = "NEW_METHOD";
	protected final static String DELETE_METHOD_BUTTON_ACTION = "DELETE_METHOD";
	
	/** Header for the methods tab. */
	protected static String METHODS_TAB_HEADER = Messages.getString("EntityPropertiesDialog.METHODS_TAB_HEADER");
	
	/** Headers for the method table. */
	protected final static String[] METHODS_TABLE_COLUMNS = {
		// Header for method name column.
		Messages.getString("EntityPropertiesDialog.METHOD_COLUMN_NAME"),
		// Header for method target column.
		Messages.getString("EntityPropertiesDialog.METHOD_TARGET_COLUMN_NAME")};
	
	/**
	 * Action performed: User clicked the 'new method' button.
	 *
	 */
	private void newMethodButtonClicked() {
		// Add new row to method table.
		Vector<String> row = new Vector<String>();
		row.add("");
		row.add("");
		getModel().addRow(row);
	}
	
	/**
	 * Action performed: User clicked the 'delete method' button.
	 */
	private void deleteMethodButtonClicked() {
		// Stop cell editing in the table.
		if ( getTable().isEditing() )
			getTable().getCellEditor().cancelCellEditing();
		// Delete selected row from methods table.
		if ( getTable().getSelectedRow() < 0 ) return;
		getModel().removeRow(getTable().getSelectedRow());
	}
	
	
	/**
	 * This method is called when an action is performed and it
	 * calls the corresponding methods to handle the action. 
	 */
	public void action(String cmd) {
		if ( NEW_METHOD_BUTTON_ACTION.equals(cmd) ) {
			newMethodButtonClicked(); return;
		}
		
		if ( DELETE_METHOD_BUTTON_ACTION.equals(cmd) ) {
			deleteMethodButtonClicked(); return;
		}
	}
	
	
	private final static Logger logger = Logger.getLogger("ucot.gui");
	
	public MethodsPanel(){
		super(METHODS_TAB_HEADER,METHODS_TABLE_COLUMNS);
		// Create required buttons.
		addButton(NEW_METHOD_BUTTON,NEW_METHOD_BUTTON_ACTION);
		addButton(DELETE_METHOD_BUTTON,DELETE_METHOD_BUTTON_ACTION);

		
	}
	
	
	
	public void load(AnalyzeModel analyzeModel,String loadEntityName) {
		super.load(analyzeModel,loadEntityName);
		updateCellEditor(analyzeModel, loadEntityName,1,true);
		Set<String> methods = analyzeModel.getEditor().getMethodNames(loadEntityName);
		for (String method : methods ) {
			Set<String> targets = analyzeModel.getEditor().
			getEntitiesInfluencedByMethod(loadEntityName, method);
			if (targets.size() > 0){
				for (String target: targets ) {
					Vector<String> row = new Vector<String>();
					row.addElement(method);
					if ( target.equals(loadEntityName) ) target = SELF_POINTER_NAME;
					row.addElement(target);
					getModel().addRow(row);
				}
			} else {
				Vector<String> row = new Vector<String>();
				row.addElement(method);
				row.addElement("");
				getModel().addRow(row);        		
			}
		}
		
	}
	
	public void save(AnalyzeModel analyzeModel,String saveEntityName) {
		
		if ( getTable().isEditing() ) 
			getTable().getCellEditor().stopCellEditing();
		
		
		ModelEditor editor = analyzeModel.getEditor(); 
		// Initialize old methods and current methods sets.
		Map<String, Set<String>> methodEntityRelations = new HashMap<String, Set<String>>();
		Set<String> oldMethods = analyzeModel.getEditor().getMethodNames(saveEntityName);
		Set<String> currentMethods = new HashSet<String>();
		for ( int i = 0; i < getModel().getRowCount(); i++ ) {
			String methodName = ((String) getModel().getValueAt(i,0)).trim();
			String entityName = ((String) getModel().getValueAt(i,1)).trim();
			if ( entityName.equals(SELF_POINTER_NAME) ) entityName = saveEntityName;
			
			// Check wether row has empty cells or not.                    
			if ( methodName.length() > 0 ) {
				currentMethods.add(methodName);
				
				Set<String> list = methodEntityRelations.get(methodName);
				if ( null == list ){ 
					list = new HashSet<String>();
					methodEntityRelations.put(methodName,list);
				}
				if ( entityName.length() > 0 ){
					list.add(entityName);
				}
			}
		}
		
		// Calculate new methods, deleted methods and changed methods.
		Set<String> newMethods = new HashSet<String>();
		newMethods.addAll(currentMethods);
		newMethods.removeAll(oldMethods);
		Set<String> deletedMethods = new HashSet<String>();
		deletedMethods.addAll(oldMethods);
		deletedMethods.removeAll(currentMethods);
		Set<String> possiblyChangedMethods = new HashSet<String>();
		possiblyChangedMethods.addAll(oldMethods);
		possiblyChangedMethods.removeAll(deletedMethods);
		
		if ( newMethods.size() > 0 || deletedMethods.size() > 0 ||
				possiblyChangedMethods.size() > 0 ) {
			//owner.setChanged(true);
		}
		
		
		// Add new methods to the analyze model.
		for ( String methodName : newMethods ) {
			try {
				editor.addMethod(saveEntityName, methodName);
				Set<String> currentEntities = methodEntityRelations.get(methodName);
				for ( String entity : currentEntities ) {
					editor.addEntityInfluenceByMethod(saveEntityName,methodName, entity);
				}
			} catch (AnalyzeModelException e) {
				e.printStackTrace();
			}
		}
		
		// Remove non-existing method relationships from the analyze model.
		for ( String methodName : deletedMethods ) {
			try {
				editor.removeMethod(saveEntityName, methodName);
			} catch (NoSuchEntityException e) {
				e.printStackTrace();
			}
		}
		
		Set<String> changedMethods = new HashSet<String>();
		// Update changed methods from the analyze model.
		for ( String methodName : possiblyChangedMethods ) {
			Set<String> oldEntities = editor.getEntitiesInfluencedByMethod(saveEntityName,
					methodName);
			Set<String> currentEntities = methodEntityRelations.get(methodName);
			Set<String> missingEntities = new HashSet<String>();
			missingEntities.addAll(currentEntities);
			missingEntities.removeAll(oldEntities);
			
			Set<String> unnecessaryEntities = new HashSet<String>();
			unnecessaryEntities.addAll(oldEntities);
			unnecessaryEntities.removeAll(currentEntities);
			for(String entity : missingEntities){
				try {
					editor.addEntityInfluenceByMethod(saveEntityName, methodName, 
							entity);
				} catch (AnalyzeModelException e) {
					e.printStackTrace();
				}
			}
			
			for(String entity : unnecessaryEntities){
				try {
					editor.removeEntityInfluenceByMethod(saveEntityName, methodName, 
							entity);
				} catch (AnalyzeModelException e) {
					e.printStackTrace();
				}
			}
			if (missingEntities.size()>0 || unnecessaryEntities.size()>0) changedMethods.add(methodName);
		}
		logger.fine(Messages.formatMessage(NEW_METHODS_LOG_MESSAGE, newMethods));
		logger.fine(Messages.formatMessage(DELETED_METHODS_LOG_MESSAGE, deletedMethods));
		logger.fine(Messages.formatMessage(CHANGED_METHODS_LOG_MESSAGE, changedMethods));
		
		
	}
	
	
	
}
