package ucot.ui.gui.tree.analyzemodeltree;

import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.swing.Icon;

/**
 * <pre>public abstract class <b>AbstractEntityTreeItem</b></pre>
 * <p>This class represents all <tt>TreeItems<tt> in the
 * <tt>AnalyzeModelTrre</tt>. It gathers common functionality
 * shared among <tt>TreeItems</tt>s that represents entities
 * in different levels in the tree.</p>
 * 
 *
 */
public abstract class AbstractEntityTreeItem extends TreeItem {
    
    protected boolean expand;

    public AbstractEntityTreeItem(String name,AnalyzeTreeModel model, TreeItem parent) {
        super(name,model, parent);
        this.expand = true;
    }

    public List<TreeItem> getChildren() {
    	if (expand && getAnalyzeModel().getEditor().containsEntity(getName())){
            List<TreeItem> items = new Vector<TreeItem>();
            items.addAll( this.getChildTreeItems() );
            items.addAll( this.getAttributeTreeItems() );
            items.addAll( this.getMethodTreeItems() );
            
            return items;
        }
        else return new Vector<TreeItem>();
    }

    
    protected List<MethodTreeItem> getMethodTreeItems() {
        List<MethodTreeItem> items = new Vector<MethodTreeItem>();
        Set<String> methods = getAnalyzeModel().getEditor().getMethodNames(getName());
        for (String  method: methods) {
            MethodTreeItem methodItem = new MethodTreeItem(getName(), method, getTreeModel(),this);
            items.add(methodItem);
        }
        
        Collections.sort(items);
        Collections.reverse(items);
        
        return items;
    }
    
    protected List<AttributeTreeItem> getAttributeTreeItems() {
        List<AttributeTreeItem> items = new Vector<AttributeTreeItem>();
        Set<String> attributes = getAnalyzeModel().getEditor().getAttributeNames(getName());
        for (String attribute : attributes) {
            items.add( new AttributeTreeItem(attribute,getTreeModel(), this) );
        }
        
        Collections.sort(items);
        Collections.reverse(items);
        return items;
    }
    
    protected List<ChildTreeItem> getChildTreeItems() {
        List<ChildTreeItem> items = new Vector<ChildTreeItem>();
        Set<String> children = getAnalyzeModel().getEditor().getChildren(getName());
        for (String child : children) {
            items.add( new ChildTreeItem(child, this.getTreeModel(), this) );
        }
        
        Collections.sort(items);
        Collections.reverse(items);
        
        return items;
    }

    @Override
    public abstract Icon getIcon();
    
    
    
}
