package ucot.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;

import ucot.Messages;

public class FileTools {

	public static final String FILE_NOT_FOUND = Messages.getString("FILE_NOT_FOUND");
	public static final String FILE_NOT_READABLE = Messages.getString("FILE_NOT_READABLE");
	public static final String FILE_NOT_WRITABLE = Messages.getString("FILE_NOT_WRITABLE");
	public static final String OPERATION_FORBIDDEN = Messages.getString("OPERATION_FORBIDDEN");
	
	
	public static void copyFile(File from, File to) throws IOException {
		if ( !from.exists() ) {
			throw new IOException(Messages.formatMessage(FILE_NOT_FOUND, from));
		}
		if ( !from.isFile() ) {
			throw new IOException(OPERATION_FORBIDDEN);
		}
		if ( !from.canRead() ) {
			throw new IOException(Messages.formatMessage(FILE_NOT_READABLE, from));
		}
		
		if ( to.isDirectory() ) to = new File(to, from.getName());
		
		if ( to.exists() ) {
			if ( !to.canWrite() ) {
				throw new IOException(Messages.formatMessage(FILE_NOT_WRITABLE, to));
			}
		} else {
			File dir = new File(to.getParent());
			
			if ( !dir.isDirectory() || !dir.exists() || !dir.canWrite() ) {
				throw new IOException(Messages.formatMessage(FILE_NOT_WRITABLE, to));
			}
			
			to.createNewFile();
		}
		
		FileInputStream input = null;
		FileOutputStream output = null;
		
		try {
			try {
				input = new FileInputStream(from);
				output = new FileOutputStream(to);
				
				final int bufferSize = 5120;
				byte[] buffer = new byte[bufferSize];
				int read = 0;
				
				while ( (read = input.read(buffer, 0, bufferSize)) != -1 ) {
					output.write(buffer, 0, read);
				}
			} finally {
				if ( input != null ) input.close();
				if ( output != null ) output.close();
			}
		} catch (IOException e) {
			throw e;
		}
	}
	
	public static void copyFile(URL from, URL to) throws IOException {
		copyFile(from.getFile(), to.getFile());
	}
	
	public static void copyFile(String from, String to) throws IOException {
		copyFile(new File(from), new File(to));
	}

}
